import abc


class Sink(abc.ABC):

    @abc.abstractproperty
    def data(self):
        pass

    @abc.abstractmethod
    def collect(self, data):
        pass


class ConcreteSink(Sink):
    """A Sink implementation. It holds the Pipe data.
    
    ...
    Properties:
    - data : object
        The data loaded in memory.
        
    Methods:
    - collect(data)
        Loads data into memory.
    - __init__()
        Creates an empty sink."""

    @property
    def data(self):
        """Returns all collected data."""
        if self._data is None:
            self.get_log(self._data).warning('Sink is empty.')
        return self._data

    def collect(self, data):
        """Collects the data payload."""
        self._data = data



import abc
import easypype.command as c
import easypype.sink as s
from multiprocessing.pool import ThreadPool


class Pipe(c.Command):
    """A Command implementation that executes a list of commands.
    
    ...
    Attributes:
    - commands : List[Command]
        The commands to be executed.
    ...
    Methods:
    - do(sink : Sink)
        Executes each command listed, saving results to the sink."""

    def __init__(self):
        self.pool = ThreadPool()
        self.commands = list()

    def parallelize(self, sink: s.Sink, command: c.Command):
        arguments = [(sink, )]
        sink.collect(self.pool.starmap(command.do, arguments)[0])

    def do(self, sink: s.Sink):
        """Executes each command listed."""
        for command in self.commands:
            self.parallelize(sink, command)


class PipeBuilder(abc.ABC):

    @abc.abstractproperty
    def build(self) -> Pipe:
        pass

    @abc.abstractmethod
    def command(self, command: c.Command):
        pass


class PipeBuilderConcrete(PipeBuilder):
    """A PipeBuilder implementation.
    
    ...
    Attributes:
    - pipe : Pipe
        The Pipe object to be built.
    ...
    Methods:
    - build()
        Returns the built Pipe.
    - command(command : Command)
        Appends the command to Pipe list of commands.
    - __init__()
        Begins the building."""

    def __init__(self):
        """Initializes an empty Pipe to be built."""
        self.pipe = Pipe()

    def build(self) -> Pipe:
        """Returns Pipe instance."""
        return self.pipe

    def command(self, command: c.Command):
        """Appends command to Pipe."""
        self.pipe.commands.append(command)
        return self



from setuptools import setup, Extension
from pathlib import Path


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
  name = 'easypype',         # How you named your package folder (MyLib)
  packages = ['easypype'],   # Chose the same as "name"
  version = '1.2.0',      # Start with a small number and increase it with every change you make
  license='unlicense',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Easy way of creating pipelines',   # Give a short description about your library
  long_description = long_description,
  long_description_content_type = 'text/markdown',
  author = 'Gabriel Felix de Souza Lopes <gafelix435@gmail.com>, Alejandro Tarafa Guzmán <alejandro.guzman@looplex.com.br>',                   # Type in your name
  url = 'https://github.com/looplex/easypype/tree/main',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/looplex/easypype/archive/refs/tags/v1.2.0.tar.gz',    # I explain this later on
  keywords = ['Pipeline', 'ETL', 'transform', 'load', 'pipe'],   # Keywords that define your package best
  install_requires = [            # I get to this in a second
      ],
  classifiers = [
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Libraries :: Python Modules',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
  ],
  python_requires = '>=3.5',
)



import abc
import easypype.sink as s
import numbers as n


class Command(abc.ABC):

    @abc.abstractmethod
    def do(self, sink: s.Sink):
        pass


class Sum(Command):
    """A command implementation that sums some value to a iterable.
    
    ...
    Attributes:
    - amount : number
        The value to be summed.
    ...
    Methods:
    - __init__(amount : Number)
        Assigns the value to be summed."""

    def __init__(self, amount: n.Number):
        """Initializes the command with the given parameters."""
        self.amount = amount

    def sum(self, sink: s.Sink):
        return [i + self.amount for i in sink.data]

    def do(self, sink: s.Sink):
        return self.sum(sink)
